/************************************************************************ *********
 * Proiectarea cu Microprocesoare
 * Laboratorul 1
 * Digital I/O
 *********************************************************************************/

#include <avr\io.h>
#include <avr\interrupt.h>
#define F_CPU 16000000
#include <util/delay.h>
#include <stdio.h>
#include <stdlib.h> 
#include <string.h>
#include <avr/pgmspace.h>
#include "music.h"

#define MIN_S 85
#define MAX_S 160
#define DIFF_S 76

#include "lcd.h"

unsigned char grid[8][8] = {{0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x1f},
							{0x0,0x0,0x0,0x0,0x0,0x0,0x1f,0x1f},
							{0x0,0x0,0x0,0x0,0x0,0x1f,0x1f,0x1f},
							{0x0,0x0,0x0,0x0,0x1f,0x1f,0x1f,0x1f},
							{0x0,0x0,0x0,0x1f,0x1f,0x1f,0x1f,0x1f},
							{0x0,0x0,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f},
							{0x0,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f},
							{0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f,0x1f}};


void save_custom_chars(){

int i,j;

	for(j = 0; j < 8; j++){
		LCD_writeInstruction(0x40 + (j*8));
		
		for(i=0;i<8;i++)
				LCD_writeData(grid[j][i]);
				
		LCD_writeInstruction(0x80);
		
	}
}

int **generate_vector(int **matrix){
	
	int i,aux;

	
	for(i = 0; i < 16; i++){
		aux = rand()%16;
		if(aux < 8){
			matrix[1][i] = aux;
			matrix[0][i] = -1;
			}
		else{
			matrix[0][i] = 15 - aux - 1;
			matrix[1][i] = 7;
			}
	}

	return matrix;
}

int **read_sample(int **matrix, int index){
	
	int i,aux,k = index,f;

	
	for(i = 0; i < 16; i++, k++){
		f = pgm_read_byte(&pcm_samples[k]);
		if(f - MIN_S < 0){
			matrix[1][i] = -1;
			matrix[0][i] = -1;
		}
		else if(f - MIN_S < 0){
			matrix[1][i] = 7;
			matrix[0][i] = 7;
		}
		
		else{
			aux = (f - MIN_S) / 5;
			if(aux < 8){
				matrix[1][i] = aux;
				matrix[0][i] = -1;
				}
			else{
				matrix[0][i] = 15 - aux - 1;
				matrix[1][i] = 7;
			}
		}
			
	}

	return matrix;
}


/*	Afisare caracter custom 	*/
void printCustomChar(int **vector)
{
	/*	TODO */
	int i,j;
	
	/*	Scriere octeti caracter custom in  CGRAM */

	LCD_writeInstruction(0x01);
	
	//LCD_writeInstruction(0x02);
	for(j = 0; j < 2; j++){
		for(i=0;i<16;i++){
			
			if(vector[j][i] >= 0)
				LCD_writeData(vector[j][i]);
			else
				LCD_writeData(' ');
		
		}
		LCD_writeInstruction(0xC0);
	}	

	_delay_ms(100);
	
	
}

void load_function(){

	int i;
	LCD_print("Loading data...");
	
	LCD_writeInstruction(LCD_INSTR_nextLine);
	
	for(i=0;i<16;i++){
		LCD_writeData(0x07);
		_delay_ms(100);
	}
	
}


void adc_init_bw(void) {
	ADCSRA |= ((1<<ADPS2) | (1<<ADPS1) | (1<<ADPS0));    //16Mhz/128 = 125Khz the ADC reference clock
	ADMUX |= (1<<REFS0);                //Voltage reference from Avcc (5v)
	ADCSRA |= (1<<ADEN);                //Turn on ADC
	ADCSRA |= (1<<ADSC);                //Do an initial conversion because this one is the slowest and to ensure that everything is up and running
}
 
int read_adc_bw(uint8_t channel) {		 //Function to read an arbitrary analogic channel/pin
	int adc_value, temp;
	
	/*
	 * To do:
	 * 1. Set ADC channel
	 */
	ADMUX = channel;
	ADMUX |= (1<<REFS0);       
	
	ADCSRA |= (1<<ADSC);                //Starts a new conversion
	while(!bit_is_set(ADCSRA,ADIF));    //Wait until the conversion is done: until ADIF from ADCSRA is set to 1
	temp = ADCL; 
	adc_value = (ADCH<<8); 
	adc_value |= temp;
	
	return adc_value;
						 //Returns the ADC value of the chosen channel
}


int main()
{	
	int rand_nr,i,j,output;
	int length =  pcm_length;
	
	int **matrix = (int**)malloc(2*sizeof(int*));
	for(i = 0; i < 2; i++)
		matrix[i] = (int*)malloc(16*sizeof(int*));
	
	LCD_init();
	save_custom_chars();
	_delay_ms(300);
	
	load_function();

	LCD_writeInstruction(0x01);
	LCD_print("Press button..");
	
	while(1)
		if (!(PIND & (1 << PD6)))
		{
			break;
		}
		
	while(i < length){
	
		matrix = read_sample(matrix,i);
		printCustomChar(matrix);
		i+=16;

	}

	LCD_print("Finished!");
	while(1);
	
	return 0;
}
